@@ -22,7 +22,7 @@ use \htmlGroup;
 /*
 
   This code is part of LDAP Account Manager (http://www.ldap-account-manager.org/)
-  Copyright (C) 2003 - 2018  Roland Gruber
+  Copyright (C) 2003 - 2020  Roland Gruber
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -70,7 +70,9 @@ lam_start_session();
 setlanguage();
 
 // get password
-if (isset($_POST['passwd'])) $passwd = $_POST['passwd'];
+if (isset($_POST['passwd'])) {
+	$passwd = $_POST['passwd'];
+}
 
 // check if password was entered
 // if not: load login page
@@ -91,7 +93,9 @@ $conf = &$_SESSION['conf_config'];
 if ((!isset($_SESSION['conf_isAuthenticated']) || !($_SESSION['conf_isAuthenticated'] === $conf->getName())) && !$conf->check_Passwd($passwd)) {
 	$sessionKeys = array_keys($_SESSION);
 	for ($i = 0; $i < sizeof($sessionKeys); $i++) {
-		if (substr($sessionKeys[$i], 0, 5) == "conf_") unset($_SESSION[$sessionKeys[$i]]);
+		if (substr($sessionKeys[$i], 0, 5) == "conf_") {
+			unset($_SESSION[$sessionKeys[$i]]);
+		}
 	}
 	$_SESSION['conf_message'] = new htmlStatusMessage('ERROR', _("The password is invalid! Please try again."));
 	/** go back to login if password is invalid */
@@ -177,8 +181,8 @@ printConfigurationPageTabs(ConfigurationPageTab::GENERAL);
 
 ?>
 <input type="text" name="hiddenPreventAutocomplete" autocomplete="false" class="hidden" value="">
-<input type="password" name="hiddenPreventAutocompletePwd1" autocomplete="false" class="hidden" value="">
-<input type="password" name="hiddenPreventAutocompletePwd2" autocomplete="false" class="hidden" value="">
+<input type="password" name="hiddenPreventAutocompletePwd1" autocomplete="false" class="hidden" value="123">
+<input type="password" name="hiddenPreventAutocompletePwd2" autocomplete="false" class="hidden" value="321">
 <?php
 
 $row = new htmlResponsiveRow();
@@ -461,28 +465,46 @@ if (extension_loaded('curl')) {
 			_('None') => TwoFactorProviderService::TWO_FACTOR_NONE,
 			'privacyIDEA' => TwoFactorProviderService::TWO_FACTOR_PRIVACYIDEA,
 			'YubiKey' => TwoFactorProviderService::TWO_FACTOR_YUBICO,
+			'Duo' => TwoFactorProviderService::TWO_FACTOR_DUO,
 	);
+    if (version_compare(phpversion(), '7.2.0') >= 0) {
+        $twoFactorOptions['Webauthn'] = TwoFactorProviderService::TWO_FACTOR_WEBAUTHN;
+    }
 	$twoFactorSelect = new htmlResponsiveSelect('twoFactor', $twoFactorOptions, array($conf->getTwoFactorAuthentication()), _('Provider'), '514');
 	$twoFactorSelect->setHasDescriptiveElements(true);
 	$twoFactorSelect->setTableRowsToHide(array(
-			TwoFactorProviderService::TWO_FACTOR_NONE => array('twoFactorURL', 'twoFactorInsecure', 'twoFactorLabel',
-				'twoFactorOptional', 'twoFactorCaption', 'twoFactorClientId', 'twoFactorSecretKey'),
-			TwoFactorProviderService::TWO_FACTOR_PRIVACYIDEA => array('twoFactorClientId', 'twoFactorSecretKey')
+		TwoFactorProviderService::TWO_FACTOR_NONE => array('twoFactorURL', 'twoFactorURLs', 'twoFactorInsecure', 'twoFactorLabel',
+			'twoFactorOptional', 'twoFactorCaption', 'twoFactorClientId', 'twoFactorSecretKey', 'twoFactorAttribute', 'twoFactorDomain'),
+		TwoFactorProviderService::TWO_FACTOR_PRIVACYIDEA => array('twoFactorURLs', 'twoFactorClientId', 'twoFactorSecretKey', 'twoFactorDomain'),
+		TwoFactorProviderService::TWO_FACTOR_YUBICO => array('twoFactorURL', 'twoFactorAttribute', 'twoFactorDomain'),
+		TwoFactorProviderService::TWO_FACTOR_DUO => array('twoFactorURLs', 'twoFactorOptional', 'twoFactorInsecure', 'twoFactorLabel', 'twoFactorDomain'),
+		TwoFactorProviderService::TWO_FACTOR_WEBAUTHN => array('twoFactorURL', 'twoFactorURLs', 'twoFactorInsecure', 'twoFactorLabel',
+			'twoFactorCaption', 'twoFactorClientId', 'twoFactorSecretKey', 'twoFactorAttribute'),
 	));
 	$twoFactorSelect->setTableRowsToShow(array(
-			TwoFactorProviderService::TWO_FACTOR_PRIVACYIDEA => array('twoFactorURL', 'twoFactorInsecure', 'twoFactorLabel',
-				'twoFactorOptional', 'twoFactorCaption'),
-			TwoFactorProviderService::TWO_FACTOR_YUBICO => array('twoFactorURL', 'twoFactorInsecure', 'twoFactorLabel',
-				'twoFactorOptional', 'twoFactorCaption', 'twoFactorClientId', 'twoFactorSecretKey'),
+		TwoFactorProviderService::TWO_FACTOR_PRIVACYIDEA => array('twoFactorURL', 'twoFactorInsecure', 'twoFactorLabel',
+			'twoFactorOptional', 'twoFactorCaption', 'twoFactorAttribute'),
+		TwoFactorProviderService::TWO_FACTOR_YUBICO => array('twoFactorURLs', 'twoFactorInsecure', 'twoFactorLabel',
+			'twoFactorOptional', 'twoFactorCaption', 'twoFactorClientId', 'twoFactorSecretKey'),
+		TwoFactorProviderService::TWO_FACTOR_DUO => array('twoFactorURL', 'twoFactorLabel',
+			'twoFactorCaption', 'twoFactorClientId', 'twoFactorSecretKey', 'twoFactorAttribute'),
+		TwoFactorProviderService::TWO_FACTOR_WEBAUTHN => array('twoFactorDomain', 'twoFactorOptional')
 	));
 	$row->add($twoFactorSelect, 12);
+	$twoFactorAttribute = new htmlResponsiveInputField(_("User name attribute"), 'twoFactorAttribute', $conf->getTwoFactorAuthenticationAttribute(), '528');
+	$row->add($twoFactorAttribute, 12);
 	$twoFactorUrl = new htmlResponsiveInputField(_("Base URL"), 'twoFactorURL', $conf->getTwoFactorAuthenticationURL(), '515');
 	$twoFactorUrl->setRequired(true);
 	$row->add($twoFactorUrl, 12);
+	$twoFactorUrl = new htmlResponsiveInputTextarea('twoFactorURLs', $conf->getTwoFactorAuthenticationURL(), '80', '4', _("Base URLs"), '515a');
+	$twoFactorUrl->setRequired(true);
+	$row->add($twoFactorUrl, 12);
 	$twoFactorClientId = new htmlResponsiveInputField(_("Client id"), 'twoFactorClientId', $conf->getTwoFactorAuthenticationClientId(), '524');
 	$row->add($twoFactorClientId, 12);
-	$twoFactorSecretKey = new htmlResponsiveInputField(_("Secret key"), 'twoFactorSecretKey', $conf->getTwoFactorAuthenticationSecretKey(), '525');
+	$twoFactorSecretKey = new htmlResponsiveInputField(_("Secret key"), 'twoFactorSecretKey', $conf->getTwoFactorAuthenticationSecretKey(), '528');
 	$row->add($twoFactorSecretKey, 12);
+	$twoFactorDomain = new htmlResponsiveInputField(_("Domain"), 'twoFactorDomain', $conf->getTwoFactorAuthenticationDomain(), '529');
+	$row->add($twoFactorDomain, 12);
 	$twoFactorLabel = new htmlResponsiveInputField(_("Label"), 'twoFactorLabel', $conf->getTwoFactorAuthenticationLabel(), '517');
 	$row->add($twoFactorLabel, 12);
 	$row->add(new htmlResponsiveInputCheckbox('twoFactorOptional', $conf->getTwoFactorAuthenticationOptional(), _('Optional'), '519'), 12);
@@ -496,9 +518,9 @@ if (extension_loaded('curl')) {
 // new password
 $row->add(new htmlSubTitle(_("Profile password"), '../../graphics/keyBig.png', null, true), 12);
 $password1 = new htmlResponsiveInputField(_("New password"), 'passwd1', null, '212');
-$password1->setIsPassword(true);
+$password1->setIsPassword(true, false, true);
 $password2 = new htmlResponsiveInputField(_("Reenter password"), 'passwd2');
-$password2->setIsPassword(true);
+$password2->setIsPassword(true, false, true);
 $password2->setSameValueFieldID('passwd1');
 $row->add($password1, 12);
 $row->add($password2, 12);
@@ -534,7 +556,6 @@ parseHtml(null, $buttonContainer, array(), false, $tabindex, 'user');
  */
 function checkInput() {
 	$conf = &$_SESSION['conf_config'];
-	$types = $conf->get_ActiveTypes();
 
 	// check new preferences
 	$errors = array();
@@ -652,15 +673,33 @@ function checkInput() {
 	$chmodOwner = 0;
 	$chmodGroup = 0;
 	$chmodOther = 0;
-	if (isset($_POST['chmod_owr']) && ($_POST['chmod_owr'] == 'on')) $chmodOwner += 4;
-	if (isset($_POST['chmod_oww']) && ($_POST['chmod_oww'] == 'on')) $chmodOwner += 2;
-	if (isset($_POST['chmod_owe']) && ($_POST['chmod_owe'] == 'on')) $chmodOwner += 1;
-	if (isset($_POST['chmod_grr']) && ($_POST['chmod_grr'] == 'on')) $chmodGroup += 4;
-	if (isset($_POST['chmod_grw']) && ($_POST['chmod_grw'] == 'on')) $chmodGroup += 2;
-	if (isset($_POST['chmod_gre']) && ($_POST['chmod_gre'] == 'on')) $chmodGroup += 1;
-	if (isset($_POST['chmod_otr']) && ($_POST['chmod_otr'] == 'on')) $chmodOther += 4;
-	if (isset($_POST['chmod_otw']) && ($_POST['chmod_otw'] == 'on')) $chmodOther += 2;
-	if (isset($_POST['chmod_ote']) && ($_POST['chmod_ote'] == 'on')) $chmodOther += 1;
+	if (isset($_POST['chmod_owr']) && ($_POST['chmod_owr'] == 'on')) {
+		$chmodOwner += 4;
+	}
+	if (isset($_POST['chmod_oww']) && ($_POST['chmod_oww'] == 'on')) {
+		$chmodOwner += 2;
+	}
+	if (isset($_POST['chmod_owe']) && ($_POST['chmod_owe'] == 'on')) {
+		$chmodOwner += 1;
+	}
+	if (isset($_POST['chmod_grr']) && ($_POST['chmod_grr'] == 'on')) {
+		$chmodGroup += 4;
+	}
+	if (isset($_POST['chmod_grw']) && ($_POST['chmod_grw'] == 'on')) {
+		$chmodGroup += 2;
+	}
+	if (isset($_POST['chmod_gre']) && ($_POST['chmod_gre'] == 'on')) {
+		$chmodGroup += 1;
+	}
+	if (isset($_POST['chmod_otr']) && ($_POST['chmod_otr'] == 'on')) {
+		$chmodOther += 4;
+	}
+	if (isset($_POST['chmod_otw']) && ($_POST['chmod_otw'] == 'on')) {
+		$chmodOther += 2;
+	}
+	if (isset($_POST['chmod_ote']) && ($_POST['chmod_ote'] == 'on')) {
+		$chmodOther += 1;
+	}
 	$chmod = $chmodOwner . $chmodGroup . $chmodOther;
 	if (!$conf->set_scriptrights($chmod)) {
 		$errors[] = array("ERROR", _("Script rights are invalid!"));
@@ -668,6 +707,16 @@ function checkInput() {
 	$conf->setScriptUserName($_POST['scriptuser']);
 	$conf->setScriptSSHKey($_POST['scriptkey']);
 	$conf->setScriptSSHKeyPassword($_POST['scriptkeypassword']);
+	if (!empty($_POST['scriptkey'])) {
+		include_once '../../lib/remote.inc';
+		$remote = new \LAM\REMOTE\Remote();
+		try {
+			$remote->loadKey($conf->getScriptSSHKey(), $conf->getScriptSSHKeyPassword());
+		}
+		catch (\LAMException $e) {
+			$errors[] = array('ERROR', _('SSH key file'), $e->getTitle());
+		}
+	}
 	// tool settings
 	$tools = getTools();
 	$toolSettings = array();
@@ -686,9 +735,15 @@ function checkInput() {
 	// 2-factor
 	if (extension_loaded('curl')) {
 		$conf->setTwoFactorAuthentication($_POST['twoFactor']);
-		$conf->setTwoFactorAuthenticationURL($_POST['twoFactorURL']);
+		if ($_POST['twoFactor'] === TwoFactorProviderService::TWO_FACTOR_YUBICO) {
+			$conf->setTwoFactorAuthenticationURL($_POST['twoFactorURLs']);
+		}
+		else {
+			$conf->setTwoFactorAuthenticationURL($_POST['twoFactorURL']);
+		}
 		$conf->setTwoFactorAuthenticationClientId($_POST['twoFactorClientId']);
 		$conf->setTwoFactorAuthenticationSecretKey($_POST['twoFactorSecretKey']);
+		$conf->setTwoFactorAuthenticationDomain($_POST['twoFactorDomain']);
 		$conf->setTwoFactorAuthenticationInsecure(isset($_POST['twoFactorInsecure']) && ($_POST['twoFactorInsecure'] == 'on'));
 		$conf->setTwoFactorAuthenticationLabel($_POST['twoFactorLabel']);
 		$conf->setTwoFactorAuthenticationOptional(isset($_POST['twoFactorOptional']) && ($_POST['twoFactorOptional'] == 'on'));
