@@ -18,7 +18,7 @@ use \htmlDiv;
 
   This code is part of LDAP Account Manager (http://www.ldap-account-manager.org/)
   Copyright (C) 2003 - 2006  Michael Duergner
-                2005 - 2018  Roland Gruber
+                2005 - 2020  Roland Gruber
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -106,7 +106,7 @@ if(isset($_COOKIE["lam_default_profile"]) && in_array($_COOKIE["lam_default_prof
 // Reload loginpage after a profile change
 if(isset($_GET['useProfile']) && in_array($_GET['useProfile'], $profiles)) {
 	logNewMessage(LOG_DEBUG, "Change server profile to " . $_GET['useProfile']);
-	$_SESSION['config'] = new LAMConfig($_GET['useProfile']); // Recreate the config object with the submited
+	$_SESSION['config'] = new LAMConfig($_GET['useProfile']); // Recreate the config object with the submitted
 }
 // Load login page
 elseif (!empty($default_Profile) && in_array($default_Profile, $profiles)) {
@@ -164,6 +164,10 @@ $_SESSION['header'] = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitiona
 $_SESSION['header'] .= "<html>\n<head>\n";
 $_SESSION['header'] .= "<meta http-equiv=\"content-type\" content=\"text/html; charset=" . $encoding . "\">\n";
 $_SESSION['header'] .= "<meta http-equiv=\"pragma\" content=\"no-cache\">\n		<meta http-equiv=\"cache-control\" content=\"no-cache\">";
+$manifestUrl = str_replace('/templates/login.php', '', getCallingURL());
+$manifestUrl = preg_replace('/http(s)?:\\/\\/([^\\/])+/', '', $manifestUrl);
+$manifestUrl = preg_replace('/\\?.*/', '', $manifestUrl);
+$_SESSION['header'] .= '<link rel="manifest" href="' . $manifestUrl . '/templates/manifest.php" crossorigin="use-credentials">';
 
 /**
 * Displays the login window.
@@ -218,12 +222,8 @@ function display_LoginPage($licenseValidator, $error_message) {
 			<tr>
 				<td align="left" height="30" width="34%">
 					<a class="lamLogo" href="http://www.ldap-account-manager.org/" target="new_window">
-					LAM
 					<?php
-						if (isLAMProVersion()) {
-							echo 'Pro ';
-						}
-						echo ' - ' . LAMVersion();
+						echo getLAMVersionText();
 					?>
 					</a>
 				</td>
@@ -556,7 +556,7 @@ if(isset($_POST['checklogin'])) {
 		$searchLDAP->close();
 	}
 	// try to connect to LDAP
-	$result = $_SESSION['ldap']->connect($username, $password); // Connect to LDAP server for verifing username/password
+	$result = $_SESSION['ldap']->connect($username, $password); // Connect to LDAP server for verifying username/password
 	if($result === 0) {// Username/password correct. Do some configuration and load main frame.
 		$_SESSION['loggedIn'] = true;
 		// set security settings for session
@@ -572,12 +572,6 @@ if(isset($_POST['checklogin'])) {
 		}
 		else {
 			$_SESSION['2factorRequired'] = true;
-			if (($_SESSION['config']->getLoginMethod() == LAMConfig::LOGIN_SEARCH) && ($_SESSION['config']->getHttpAuthentication() == 'true')) {
-				$_SESSION['user2factor'] = $_SERVER['PHP_AUTH_USER'];
-			}
-			else {
-				$_SESSION['user2factor'] = $_POST['username'];
-			}
 			metaRefresh("./login2Factor.php");
 		}
 		die();
