@@ -9,10 +9,9 @@ use \htmlSelect;
 use \htmlInputField;
 use \htmlButton;
 /*
-$Id$
 
   This code is part of LDAP Account Manager (http://www.ldap-account-manager.org/)
-  Copyright (C) 2017  Roland Gruber
+  Copyright (C) 2017 - 2019  Roland Gruber
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -46,13 +45,8 @@ startSecureSession();
 setlanguage();
 
 $config = $_SESSION['config'];
-$ldap = $_SESSION['ldap'];
-$credentials = $ldap->decrypt_login();
-$password = $credentials[1];
-$user = $_SESSION['user2factor'];
-if (get_preg($user, 'dn')) {
-	$user = extractRDNValue($user);
-}
+$password = $_SESSION['ldap']->getPassword();
+$user = $_SESSION['ldap']->getUserName();
 
 // get serials
 try {
@@ -72,7 +66,6 @@ $twoFactorLabel = empty($twoFactorLabelConfig) ? _('PIN+Token') : $twoFactorLabe
 if (sizeof($serials) == 0) {
 	if ($config->getTwoFactorAuthenticationOptional()) {
 		unset($_SESSION['2factorRequired']);
-		unset($_SESSION['user2factor']);
 		metaRefresh("main.php");
 		die();
 	}
@@ -91,10 +84,10 @@ if (isset($_POST['logout'])) {
 	exit();
 }
 
-if (isset($_POST['submit'])) {
-	$twoFactorInput = $_POST['2factor'];
-	$serial = $_POST['serial'];
-	if (empty($twoFactorInput) || !in_array($serial, $serials)) {
+if (isset($_POST['submit']) || isset($_POST['sig_response'])) {
+	$twoFactorInput = isset($_POST['2factor']) ? $_POST['2factor'] : null;
+	$serial = isset($_POST['serial']) ? $_POST['serial'] : null;
+	if (!$provider->hasCustomInputForm() && (empty($twoFactorInput) || !in_array($serial, $serials))) {
 		$errorMessage = _(sprintf('Please enter "%s".', $twoFactorLabel));
 	}
 	else {
@@ -107,7 +100,6 @@ if (isset($_POST['submit'])) {
 		}
 		if ($twoFactorValid) {
 			unset($_SESSION['2factorRequired']);
-			unset($_SESSION['user2factor']);
 			metaRefresh("main.php");
 			die();
 		}
@@ -140,7 +132,7 @@ printJsIncludes('..');
 
 	<br><br>
 
-	<form enctype="multipart/form-data" action="login2Factor.php" method="post" autocomplete="off">
+	<form id="2faform" enctype="multipart/form-data" action="login2Factor.php" method="post" autocomplete="off">
 <?php
 echo $config->getTwoFactorAuthenticationCaption();
 
@@ -156,21 +148,31 @@ echo $config->getTwoFactorAuthenticationCaption();
 		$row->add(new \htmlStatusMessage('ERROR', $errorMessage), 12);
 		$row->add(new htmlSpacer('1em', '1em'), 12);
 	}
-	// serial
-	$row->add(new htmlOutputText(_('Serial number')), 12, 12, 12, 'text-left');
-	$serialSelect = new htmlSelect('serial', $serials);
-	$row->add($serialSelect, 12);
-	// token
-	$row->add(new htmlOutputText($twoFactorLabel), 12, 12, 12, 'text-left');
-	$twoFactorInput = new htmlInputField('2factor', '');
-	$twoFactorInput->setFieldSize(null);
-	$twoFactorInput->setIsPassword(true);
-	$row->add($twoFactorInput, 12);
+
+	if (!$provider->hasCustomInputForm()) {
+		// serial
+		$row->add(new htmlOutputText(_('Serial number')), 12, 12, 12, 'text-left');
+		$serialSelect = new htmlSelect('serial', $serials);
+		$row->add($serialSelect, 12);
+		// token
+		$row->add(new htmlOutputText($twoFactorLabel), 12, 12, 12, 'text-left');
+		$twoFactorInput = new htmlInputField('2factor', '');
+		$twoFactorInput->setFieldSize(null);
+		$twoFactorInput->setIsPassword(true);
+		$row->add($twoFactorInput, 12);
+	}
+	else {
+		$provider->addCustomInput($row, $user);
+	}
+
+	// buttons
 	$row->add(new htmlSpacer('1em', '1em'), 12);
-	$submit = new htmlButton('submit', _("Submit"));
-	$submit->setCSSClasses(array('fullwidth'));
-	$row->add($submit, 12, 12, 12, 'fullwidth');
-	$row->add(new htmlSpacer('0.5em', '0.5em'), 12);
+	if ($provider->isShowSubmitButton()) {
+		$submit = new htmlButton('submit', _("Submit"));
+		$submit->setCSSClasses(array('fullwidth'));
+		$row->add($submit, 12, 12, 12, 'fullwidth');
+		$row->add(new htmlSpacer('0.5em', '0.5em'), 12);
+	}
 	$logout = new htmlButton('logout', _("Cancel"));
 	$logout->setCSSClasses(array('fullwidth'));
 	$row->add($logout, 12);
@@ -188,7 +190,9 @@ echo $config->getTwoFactorAuthenticationCaption();
 
 	<script type="text/javascript">
 		myElement = document.getElementsByName('2factor')[0];
-		myElement.focus();
+		if (myElement) {
+			myElement.focus();
+		}
 	</script>
 </body>
 </html>
